@@ -9,7 +9,7 @@ use List::Util 1.45 qw( uniq );
 
 BEGIN {
 	our $AUTHORITY = 'cpan:TOBYINK';
-	our $VERSION   = '0.016002';
+	our $VERSION   = '0.017000';
 	
 	if ( eval { require Types::Standard; 1 } ) {
 		Types::Standard->import(
@@ -42,15 +42,18 @@ sub import {
 	$package    ||= our($SETUP_FOR) || caller;
 	$methodname ||= 'new';
 	
+	$META{$package} ||= { package => $package };
+	
 	if (our $REDEFINE) {
 		no warnings 'redefine';
 		install_constructor("$package\::$methodname");
+		install_BUILDALL("$package\::BUILDALL");
 	}
 	else {
 		install_constructor("$package\::$methodname");
+		install_BUILDALL("$package\::BUILDALL");
 	}
-	
-	$META{$package} ||= { package => $package };
+
 	inheritance_stuff($package);
 	
 	do {
@@ -121,7 +124,7 @@ sub import {
 			_croak("Required attribute $name cannot have undef init_arg");
 		}
 		
-		my @unknown_keys = sort grep !/\A(isa|required|is|default|builder|coerce|init_arg|trigger|weak_ref|alias)\z/, keys %spec;
+		my @unknown_keys = sort grep !/\A(isa|required|is|default|builder|coerce|init_arg|trigger|weak_ref|alias|slot_initializer)\z/, keys %spec;
 		if ( @unknown_keys ) {
 			_croak("Unknown keys in spec: %d", join ", ", @unknown_keys);
 		}
@@ -164,6 +167,10 @@ sub import {
 			$meta_attribute{aliases} = [ $spec{alias} ];
 		}
 		
+		if ( is_CodeRef $spec{slot_initializer} ) {
+			$meta_attribute{slot_initializer} = $spec{slot_initializer};
+		}
+		
 		# Add new attribute
 		push @{ $META{$package}{params} ||= [] }, \%meta_attribute;
 	}
@@ -325,6 +332,7 @@ sub _build_flags {
 	$flags |= XSCON_FLAG_HAS_TRIGGER           if $spec->{trigger};
 	$flags |= XSCON_FLAG_WEAKEN                if $spec->{weak_ref};
 	$flags |= XSCON_FLAG_HAS_ALIASES           if $spec->{alias};
+	$flags |= XSCON_FLAG_HAS_SLOT_INITIALIZER  if $spec->{slot_initializer};
 	
 	my $has_common_default = 0;
 	if ( exists $spec->{default} and !defined $spec->{default} ) {
@@ -370,12 +378,32 @@ sub inheritance_stuff {
 	
 	require( $] >= 5.010 ? "mro.pm" : "MRO/Compat.pm" );
 	
-	my @isa = reverse @{ mro::get_linear_isa($package) };
-	pop @isa;  # discard $package itself
+	my @isa = @{ mro::get_linear_isa($package) };
+	shift @isa;  # discard $package itself
 	return unless @isa;
 	
-	my @attrs;
 	for my $parent ( @isa ) {
+		no strict 'refs';
+		# Moo will sometimes not have a constructor in &{"${parent}::new"}
+		# when by all that is good and holy, it should.
+		if ( $INC{'Moo.pm'} and $Moo::MAKERS{$parent} ) {
+			Moo->_constructor_maker_for( $parent )->install_delayed;
+			Sub::Defer::undefer_sub( \&{"${parent}::new"} );
+		}
+		if ( defined &{"${parent}::new"} ) {
+			if ( not $META{$parent} ) {
+				# We are inheriting from a foreign class.
+				$META{$package}{foreignclass}         = $parent;
+				$META{$package}{foreignconstructor}   = \&{"${parent}::new"};
+				$META{$package}{foreignbuildall}      = $parent->can('BUILDALL');
+				$META{$package}{foreignbuildargs}     = $parent->can('BUILDARGS');
+			}
+			last;
+		}
+	}
+	
+	my @attrs;
+	for my $parent ( reverse @isa ) {
 		my $p_attrs = $META{$parent}{params} or next;
 		push @attrs, @$p_attrs;
 	}
@@ -438,6 +466,9 @@ sub populate_build {
 
 sub get_metadata {
 	my $klass = ref($_[0]) || $_[0];
+	$META{$klass}{buildargs}        ||= $klass->can('BUILDARGS');
+	$META{$klass}{foreignbuildargs} ||= $klass->can('FOREIGNBUILDARGS')
+		if $META{$klass}{foreignconstructor} && !$META{$klass}{foreignbuildall};
 	return $META{$klass};
 }
 
@@ -689,7 +720,28 @@ Supports C<weak_ref> like L<Moose> and L<Moo>.
 
 =item *
 
-Supports Moose/Moo/Class::Tiny-style C<BUILD> methods.
+Custom slot initializers.
+
+  use Class::XSConstructor foo => {
+    slot_initializer => sub {
+      my ( $object, $value ) = @_;
+      $object->{FOO} = $value; 
+    },
+  };
+
+One of the jobs that the constructor obviously does is insert each value
+into the relevant slots in the object's underlying hash. In very rare
+cases, you might want to provide a callback to do this. A use case might
+be if you have particular attributes that you wish to store using inside-out
+object techniques, or encrypt before storing.
+
+Obviously your accessors will also need to be written to be able to access
+the value from wherever you've stored it.
+
+=item * 
+
+Supports Moose/Moo/Class::Tiny-style C<BUILD>, C<BUILDALL>, C<BUILDARGS>,
+and C<FOREIGNBUILDARGS> methods.
 
 Including C<< __no_BUILD__ >>.
 
